const fs = require('fs');

/**
 * Write some data to a file
 *
 * @param {string} filename
 * @param {*} data
 * @param {boolean} [silent = false]
 * @returns {Promise.<boolean>} success
 */
function write(filename, data, silent = false) {
  return new Promise((resolve, reject) => {
    fs.writeFile(filename, data, err => {
      if (silent) {
        return err ? resolve(false) : resolve(true);
      }

      return err ? reject(err) : resolve(true);
    });
  });
}

/**
 * Read some data from a file
 *
 * @param {string} filename
 * @param {string} [encoding='utf8']
 * @param {boolean} [silent=false]
 * @returns {Promise.<string>} data
 */
function read(filename, encoding = 'utf8', silent = false) {
  return new Promise((resolve, reject) => {
    fs.readFile(filename, encoding, (err, data) => {
      if (silent) {
        return err ? resolve(null) : resolve(data);
      }

      return err ? reject(err) : resolve(data);
    });
  });
}

/**
 * Read JSON data from a file
 *
 * @param {string} filename
 * @param {boolean} [silent=false]
 * @returns {Promise.<Object>} data
 */
function readJSON(filename, silent = false) {
  return new Promise((resolve, reject) => {
    read(filename, 'utf8', false)
      .then(data => {
        try {
          return resolve(JSON.parse(data));
        } catch (err) {
          return silent ? resolve({}) : reject(err);
        }
      })
      .catch(err => (silent ? resolve({}) : reject(err)));
  });
}

/**
 *
 * @param {string} filename
 * @param {string} [separator = ',']
 * @param {Array.<string>} [header = null]
 * @param {boolean} silent
 * @returns {Promise.<Array.<Object>>} data
 */
function readCSV(filename, separator = ',', header = null, silent = false) {
  return new Promise((resolve, reject) => {
    read(filename, 'utf8', false)
      .then(data => {
        try {
          const res = [];

          const lines = data.split('\n');

          for (let i = 0; i < lines.length; i++) {
            const line = lines[i];

            if (i == 0 && header == null) {
              header = line.split(separator);
            } else {
              const record = {};

              line
                .split(separator)
                .forEach((item, i) => (record[header[i]] = item));

              res.push(record);
            }
          }

          return resolve(res);
        } catch (err) {
          return silent ? resolve([]) : reject(err);
        }
      })
      .catch(err => (silent ? resolve([]) : reject(err)));
  });
}

module.exports = {
  write,
  read,
  readJSON,
  readCSV
};
